# *************************************************************************
#	File Name:		xwiard.tcl
# Package Name:	XWizard
# Version:      2.0
# Date:         May 26, 2002
# Description:	This wizard generates a C/C++ interface of loadable 
#		            object/string extension for Windows/Unix (Linux).
#		            Modify wizdata.tcl to define your commands.
#	Reference:    http://www.neatware.com
# Copyright (c) 1999 Neatware. All rights reserved.
# ************************************************************************

package provide XWizard 2.0

proc Begin {{filename {}}} {
	global g_fid
	set g_fid ""
	return

	if {[string compare $filename {}] == 0} {
		set g_fid stdout
		return
	}
	if [catch {open $filename w+} g_fid] {
		puts stderr "Can't Open File $filename!"
		set g_fid stdout
		return
	}
}

proc End {} {
	global g_fid
	return $g_fid

	if {[string compare $g_fid stdout] == 0} {
		return
	}
	close $g_fid 
}

proc quote {{s {}}} {
	global g_fid
	#puts $g_fid "$s"
	append g_fid "$s\n" 
}

proc quote1 {{s {}}} {
	global g_fid
	#puts -nonewline $g_fid "$s"
	append g_fid "$s"
}

proc newline {} {
	global g_fid
	#puts $g_fid ""
	append g_fid "\n"
}

proc DoComment {lang aplatform api} {
	global def
	if {[string compare $aplatform win] == 0} {
		set plat {Windows95, 98, 2000, NT, XP}
	} elseif {[string compare $aplatform unix] == 0} {
		set plat {Unix, Linux, Solaris, HP-UX, AIX}
	} else {
		set plat $aplatform
	}

	quote "
/*********************************************************************** 
 * File Name:    [lindex $def(PACKAGE) 0].$lang
 * Package Name: [lindex $def(PACKAGE) 0]
 * Version:      [lindex $def(PACKAGE) 1]
 * Platform:     $plat
 * Date:         [clock format [clock seconds] -format "%c"]
 * Description:  This $lang code is $api APIs for Tcl loadable extension
 *               on $plat. You can modify the $lang code.
 * Result:       Generate a loadable DLL/Shared Library template.
 **********************************************************************/" 
	newline
}
 
proc DoInclude {stub} {
	global def
	foreach item $def(STD_INCLUDE) {
		quote "#include <$item>"
	}
	newline

	if {[string compare $stub "none"] != 0} {
		quote "#define USE_TCL_STUBS"
	}
	newline

	foreach item $def(CUR_INCLUDE) {
		quote "#include \"$item\""
	}
	newline
}

proc DoDefine {platform} {
	global def

	if {[string compare $platform win] == 0} {
		quote "#ifndef DLLEXPORT"
		quote "#define $def(DLLDEFINE)"
		quote "#endif\n"
	} else {
		quote "#define DLLEXPORT"
	}
	foreach item $def(DEFINE) {
		quote "#define $item"
	}
	newline
}

proc DoGlobalVar {} {
	quote "/* add global variables */\n"
}

proc DoGlobalFunc {} {
	quote "/* add global functions */\n"
}

proc DoCPPHead {} {
	quote "extern \"C\" \{"
	newline
}

proc DoCPPEnd {} {
	quote "\}"
	newline
}

proc DoErrorResult {{item {}}} {
	set argc [lindex $item 1]
	set c ""
	for {set i 1} {$i < $argc} {incr i} {
		append c " arg$i"
	}

quote "		Tcl_WrongNumArgs (interp, 1, objv, \"usage: [lindex $item 0] $c\");" 
}

proc DoProcHeader {{item {}}} {

	set pname [lindex $item 0]
	set pnum [lindex $item 1]

	quote "int DLLEXPORT ${pname}_code (ClientData dummy,
	Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv\[\])"
	quote "\{"
}

proc DoVarDeclare {{item {}}} {
	quote "	/* declare variables */"
	quote "	Tcl_Obj *pObjRet;\n"
}

proc DoArgsCheck {{item {}}} {
	set argc [lindex $item 1]
	quote "	/* uncomment for argument checking"
	quote "	if (objc > $argc) \{"
	DoErrorResult $item;
	quote "		return TCL_ERROR;
	\}"
	quote "	*/"
	newline
}

proc DoImplementation {{item {}}} {
	quote "	/* my implementation */\n"
}

proc DoReturn {{item {}}} {
	quote "	pObjRet = Tcl_GetObjResult (interp);
	Tcl_SetStringObj (pObjRet, \"This is the result.\", -1);" 
	quote "	return TCL_OK;"
	quote "\}"
}

proc DoProcList {} {
	global def
	foreach item $def(PROCLIST) {
		DoProcHeader $item
		DoVarDeclare $item
		DoArgsCheck $item
		DoImplementation $item
		DoReturn $item
		newline
	}
}

proc Capital {{s {}}} {
	set c [string index $s 0]
	set c [string toupper $c]
	set len [string length $s]
	incr len -1
	append c [string range $s 1 $len]
	return $c
}

proc DoInitHeader {} {
	global def
	set pkgname [lindex $def(PACKAGE) 0]
	set pkgver [lindex $def(PACKAGE) 1]

	quote "int DLLEXPORT [Capital $pkgname]_Init (Tcl_Interp *interp)"
	quote "\{"
}

proc DoInitBody {stub} {
	global def

quote {	int i = 0; }

if {[string compare $stub "later"] == 0} {
quote {
	if (Tcl_InitStubs (interp, TCL_VERSION, 0) == 0) {
	  return TCL_ERROR;
	}
}
} elseif {[string compare $stub "all"] == 0} {
quote {
	if (Tcl_InitStubs (interp, "8.1", 0) == 0) {
	  return TCL_ERROR;
	}
}
}

quote {
	while (g_pProcName[i] != NULL) {
		Tcl_CreateObjCommand (interp, g_pProcName[i], g_pProcCode[i],
			(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
		i++; 
	}  
}
}

proc DoInitEnd {} {
	global def

	set name [lindex $def(NAMESPACE) 0]
	quote "	Tcl_PkgProvide (interp, \"[Capital [lindex $def(PACKAGE) 0]]\", \"[lindex $def(PACKAGE) 1]\");"

# use default export
#	if {$name != {}} {
#		quote "	Tcl_Eval (interp, \"namespace eval ::[lindex $def(NAMESPACE) 0] {namespace export *}\");"
#	}	

	quote "	return TCL_OK;"
	quote "\}"
}

proc DoInit {stub} {
	DoInitHeader
	DoInitBody $stub
	DoInitEnd
}

proc DoGlobalStatic {} {
	global def

	set ns [lindex $def(NAMESPACE) 0]
	if {$ns == {}} {
		set name "::"
	} else {
		set name "::${ns}::"
	}
	quote "static char *g_pProcName\[\] = \{"
	set count 1
	foreach item $def(PROCLIST) {
		if {[expr $count % 5] != 0} {
			quote1 "\"$name[lindex $item 0]\", "
		} else {
			quote "\"$name[lindex $item 0]\", "
		}
		incr count 1
	}
	quote "NULL\};"
	newline
}

proc DoFuncInit {} {
	global def

	quote "int (*g_pProcCode\[\]) (ClientData, Tcl_Interp *, int, Tcl_Obj *CONST \[\]) = \{"
	set count 1
	foreach item $def(PROCLIST) {
		if {[expr $count % 5] != 0} {
			quote1 "[lindex $item 0]_code, "
		} else {
			quote "[lindex $item 0]_code, "
		}
		incr count 1
	}
	quote "NULL\};"
	newline
}

proc CheckParam {lang platform api stub} {
	switch $lang {
		c 				{}
		cpp				{}
		default		{return 0}
	}
	switch $platform {
		win				{}
		unix			{}
		default		{return 0}
	}
	switch $api {
		obj				{}
		default		{return 0}
	}
	switch $stub {
		all			{}
		later		{}
		none				{}
		default		{return 0}
	}
	return 1
}

# 
# lang:			c, cpp
# platform:	win, unix
# api:			obj
#	stub:			all, later, none
#
#proc wizard {{lang c} {platform win} {api obj} {stub none}} {
proc wizard {param} {
	upvar $param argu
	global def

	if {[CheckParam $argu(lang) $argu(platform) $argu(api) $argu(stub)] == 0} {
		puts {Error. Usage: wizard [c|cpp] [win|unix] [obj] [all|later|none]}
		return
	}
	set pkg [lindex $def(PACKAGE) 0].$argu(lang)
	Begin $pkg
	DoComment $argu(lang) $argu(platform) $argu(api) 
	DoInclude $argu(stub)
	DoDefine $argu(platform)
	DoGlobalVar
	DoGlobalFunc
	if {[string compare $argu(lang) "cpp"] == 0} {
		DoCPPHead
	}
	DoProcList
	DoGlobalStatic
	DoFuncInit
	DoInit $argu(stub)	 
	if {[string compare $argu(lang) "cpp"] == 0} {
		DoCPPEnd
	}
	return [End]
	# puts "Success Generate $pkg!"
}