# -------------------------------------------------------------
#
# Name:				xWizard
# Version:			2.0
#	Description:	Generate C/C++ template for Tcl extension
#	Author: 			Chang Li, Neatware
#	License: 			Open Source BSD
#	Copyright: 		Chang Li, Neatware. All Rights Reserved.
#	Date:				May 26, 2002
#
# -------------------------------------------------------------

package require tkwizard
package require Tktable
package require XWizard

# we don't need the main window for this app...
wm withdraw .

# build the wizard
tkwizard::tkwizard .wizthing -title "xWizard" 

# initialize some data structures for my own use
.wizthing eval {
    variable wizData

    # default values
    catch {unset wizData}

    # this will contain the list of custom steps
    set wizData(steps) {}

    # this contains the wizard title
    	set wizData(title) 			"xWizard"
		set wizData(platform)		win
		set wizData(language)		cpp
		set wizData(stub)				all

		set wizData(package)			"MyPackage"
		set wizData(namespace)		"MyNamespace"
		set wizData(version)			"1.0"
		set wizData(stdinclude)	"windows.h"
		set wizData(userinclude)	"tcl.h"

		set wizData(maxRows)			500
}

# this sets up the final task of saving the wizard to 
# a file. 
bind .wizthing <<WizFinish>> {[%W namespace]::finalize}

# I want this to work, but it won't. The problem is when the
# finalize proc does "return -code break". I wish I could figure
# out how to make "%W eval finalize" synonymous with 
# "[%W namespace]::finalize" when that happens... 
#
#bind .wizthing <<WizFinish>> {[%W eval finalize]}

# this sets up a binding so we can programatically decide
# what the next step will be at runtime...
bind .wizthing <<WizNextStep>> {[%W namespace]::nextStep %W}

.wizthing step init  -layout advanced {
    variable wizData

    set c [$this widget clientArea]

    $this stepconfigure \
        -title "Welcome to xWizard" \
        -subtitle "This wizard will generate the template for Tcl C/C++ extension." \
        -pretext "Copyright (c) Chang Li, Neatware. BSD License." \
        -posttext "Click Next to continue."
}

.wizthing step step1 -layout basic {
    variable wizData

    set c [$this widget clientArea]

    $this stepconfigure \
        -title "xWizard 2.0" \
        -subtitle "Select platform, language, and stub." \
        -pretext "" \
        -posttext ""

		# language
		set w $c.language
		frame $w -borderwidth 2 -relief flat

		label $w.labelLanguage -text Language
		grid $w.labelLanguage -row 0 -column 0 -sticky w -pady 4

		set ls {cpp c}
		set row 1
		foreach e $ls {
			radiobutton $w.check$e -text $e \
				-variable [namespace current]::wizData(language) \
				-value $e -width 10 -anchor w
			grid $w.check$e -row $row -column 0 -sticky w
			incr row
		}

		# platform
		set w $c.platform
		frame $w -borderwidth 2 -relief flat

		label $w.labelPlatform -text Platform
		grid $w.labelPlatform -row 0 -column 0 -sticky w -pady 4

		set ls {win unix}
		set row 1
		foreach e $ls {
			radiobutton $w.check$e -text $e \
				-variable [namespace current]::wizData(platform) \
				-value $e -width 10 -anchor w
			grid $w.check$e -row $row -column 0 -sticky w
			incr row
		}

		# stub
		set w $c.stub
		frame $w -borderwidth 2 -relief flat

		label $w.labelStub -text Stub
		grid $w.labelStub -row 0 -column 0 -sticky w -pady 4

		set ls {all later none}
		set row 1
		foreach e $ls {
			radiobutton $w.check$e -text $e \
				-variable [namespace current]::wizData(stub) \
				-value "$e" -width 10 -anchor w
			grid $w.check$e -row $row -column 0 -sticky w
			incr row
		}

		pack $c.platform $c.language $c.stub -side left -anchor w -padx 16 -pady 0
}


.wizthing step step2 -layout basic {
    variable wizData

    set c [$this widget clientArea]

    # testing stepconfigure...
    $this stepconfigure \
        -title "Name Package, Namespace, and Command List" \
        -subtitle "When Namespace box is empty the extension is in global. Input command name into the Command List."  \
        -pretext "" \
        -posttext ""

		set w $c.package
		frame $w 
		label $w.lab -text Package -width 20
		entry $w.entry -width 44 -textvariable [namespace current]::wizData(package)
		pack $w.lab $w.entry -side left -fill x -expand true

		set w $c.namespace
		frame $w
		label $w.lab -text Namespace -width 20
		entry $w.entry -width 44 -textvariable [namespace current]::wizData(namespace)
		pack $w.lab $w.entry -side left -fill x -expand true

		set w $c.version
		frame $w
		label $w.lab -text Version -width 20
		entry $w.entry -width 44 -textvariable [namespace current]::wizData(version)
		pack $w.lab $w.entry -side left -fill x -expand true

		if {[string compare $wizData(language) "c"] == 0} {
				array set wizData {stdinclude stdio.h}
		} elseif {[string compare $wizData(platform) "win"] == 0} {
				array set wizData {stdinclude windows.h}
		} else {
				array set wizData {stdinclude stdio.h}
		}

		set w $c.stdinclude
		frame $w
		label $w.lab -text "Standard Include" -width 20
		entry $w.entry -width 44 -textvariable [namespace current]::wizData(stdinclude)
		pack $w.lab $w.entry -side left -fill x -expand true

		set w $c.curinclude
		frame $w
		label $w.lab -text "User Include" -width 20
		entry $w.entry -width 44 -textvariable [namespace current]::wizData(userinclude)
		pack $w.lab $w.entry -side left -fill x -expand true

		pack $c.package $c.namespace $c.version $c.stdinclude $c.curinclude -side top -fill x -expand true

		label $c.label -text "Command List"

		global table

		if {[catch {info exists $table(rows)}]} {

			array set table [list rows	$wizData(maxRows) cols	2 table $c.t array	t]

			# fill
			set x $table(rows)
			set y $table(cols)
			upvar #0 $table(array) f
    		for {set i -$x} {$i<$x} {incr i} {
				for {set j -$y} {$j<$y} {incr j} { 
					set f($i,0) $i
					set f($i,1) "" 
				}
    		}
		
			set f(-1,0) "NO."
			set f(-1,1) "Name"
			set f(0,1)	"hello"
		}

		table $table(table) -rows $table(rows) -cols $table(cols) \
			-variable $table(array) \
			-width 8 -height 4 \
			-titlerows 1 -titlecols 0 \
			-roworigin -1 -colorigin 0 \
			-yscrollcommand [list $c.sy set] \
			-colstretchmode last -rowstretchmode last \
			-highlightbackground gray \
			-highlightthickness 0 \
			-selectmode extended -sparsearray 0

		# $table(table) tag configure rowtag -justify left
		$table(table) configure -anchor w -bg white
		$table(table) tag configure active -bg #cccccc -relief solid
 
		scrollbar $c.sy -command [list $table(table) yview]

		pack $c.label -side top -fill x -expand true -pady 4
		pack $table(table) -side left -fill x -expand true
		pack $c.sy -side left -fill both
}

.wizthing step finish -layout advanced {
    variable wizData

    set c [$this widget clientArea]

    $this stepconfigure \
        -title "Finish" \
        -subtitle "There is now enough information to create the template; you may now view the generated code or go back to reset the settings." \
        -pretext "" \
        -posttext "When you are ready to save the code to disk, click\n\
                   on Finish." \

    button $c.show -text "Show Template Code" \
        -width 18 \
        -command [namespace code  "generateWizard"] 

    pack $c.show -side top  -pady 4 -anchor w
}

.wizthing eval {

    # this is called when the user clicks on the Finish button
    proc finalize {} {
	  		 variable wizData

        set filetypes {
            {{TCL Scripts} {.tcl}}
            {{All Files}   *}
        }

        set filename [tk_getSaveFile \
											 -initialfile $wizData(package).tcl \
                          -defaultextension .tcl \
                          -filetypes $filetypes \
                          -title "Save File" \
                          -parent .wizthing ]

        if {[string length $filename] ==0} {
            return -code break 
        }

        set fh [open $filename w]
        puts $fh [.wizthing eval {generateCode .wizard}]
        close $fh
    }

    # this is used by the various buttons on step 2
    proc manageListbox {this} {
        variable wizData

        set c [$this widget clientArea]

        set wizData(steps) [$c.lb_ get 0 end]

        set sel [$c.lb_ curselection]
        set size [$c.lb_ size]

        # if nothing is selected, nothing is enabled...
        if {[llength $sel] == 0} {
            $c.upButton   configure -state disabled
            $c.downButton configure -state disabled
            $c.delButton  configure -state disabled
            return

        } 

        # if there's something selected, the delete button will always
        # be enabled
        $c.delButton configure -state normal

        if {$size == 1} {
            # if there's only one item in the list, the up and down
            # buttons are disabled
            $c.upButton configure -state disabled
            $c.downButton configure -state disabled

        } else {
            set last [incr size -1]
            set i [lindex $sel 0]
            # if there's more than one in the list and the selected item
            # is not the first item, enable the up button
            if {$i > 0} {
                $c.upButton configure -state normal
            } else {
                $c.upButton configure -state disabled
            }

            # if there's more than one in the list and the selected item
            # is not the last, enable the down button
            if {$i < $last} {
                $c.downButton configure -state normal
            } else {
                $c.downButton configure -state disabled
            }
        }
    }
    proc manage {command wiz e lb} {
        variable wizData

        switch $command {
            "add" {
                # get the text of the step
                set text [$e get]
                if {[string length $text] == 0} {
                    bell
                    return
                }
                # check for duplicates
                set i [lsearch -exact [$lb get 0 end] $text]
                if {$i >= 0} {
                    tk_messageBox \
                        -icon "info" \
                        -type "ok" \
                        -parent $wiz \
                        -message "The name '$text' has already been\
                              used for a step. Step names must be unique."
                } else {
                    $lb insert end $text
                    $lb see end
                    $e delete 0 end
                }
            }
            "del" {
                set index [$lb curselection]
                if {[llength $index] > 0} {
                    set i [lindex $index 0]
                    $lb delete $i

                    # we want to make a new selection. This will
                    # be the same index if it exists, or the one prior
                    # if it exists... Interesting fact: this is the 
                    # first time in several years of tcl programming
                    # that I learned [$lb index end] returns the number
                    # of items in the listbox rather than the index of
                    # the last item. Feh. 
                    set last [expr {[$lb index end] -1}]
                    if {$i > $last} {
                        incr i -1
                    }
                    $lb selection anchor $i
                    $lb selection set $i
                }
            }
            "up" {
                set y1 [lindex [$lb curselection] 0]
                set y2 [expr {$y1 -1}]
                set item [$lb get $y1]
                $lb delete $y1
                $lb insert $y2 $item
                $lb selection anchor $y2
                $lb selection set $y2
            }
            "down" {
                set y1 [lindex [$lb curselection] 0]
                set y2 [expr {$y1 +1}]
                set item [$lb get $y1]
                $lb delete $y1
                $lb insert $y2 $item
                $lb selection anchor $y2
                $lb selection set $y2
            }
        }

        set wizData(steps) [$lb get 0 end]
    }

    proc nextStep {this} {

        variable wizData

        set currentStep [$this cget -step]

        # if the current step is step2, we need to reconfigure
        # the wizard to have a new step for each step of the 
        # wizard being created
        if {$currentStep == "step2"} {

            set order [list init step1 step2]
            for {set i 0} "\$i < [llength $wizData(steps)]" {incr i} {
                $this step newStep-$i -layout basic [list stepConfig $this $i]
                lappend order newStep-$i
            }
            lappend order finish
            eval $this order $order
            $this configure -nextstep [lindex $order 3]
        }
    }

    proc stepConfig {this index} {
        variable wizData

        set stepName [lindex $wizData(steps) $index]

        set c [$this widget clientArea]

        $this stepconfigure \
            -title "Step Configuration: $stepName" \
            -subtitle "Use this form to configure step-specific information."  \
            -pretext "" \
            -posttext ""

        label $c.layoutLabel -text "Layout:" -anchor w
        tk_optionMenu $c.layout [namespace current]::wizData(layout,$index) Basic Advanced
        $c.layout configure -width 8

        label $c.titleLabel -text "Title:" -anchor w
        entry $c.titleEntry \
            -width 32 \
            -textvariable [namespace current]::wizData(title,$index)
        label $c.subtitleLabel -text "SubTitle:" -anchor w
        entry $c.subtitleEntry \
            -width 32 \
            -textvariable [namespace current]::wizData(subtitle,$index)

        grid $c.layoutLabel   -row 0 -column 0               -sticky w
        grid $c.layout        -row 0 -column 1               -sticky w
        grid $c.titleLabel    -row 2 -column 0 -columnspan 2 -sticky w
        grid $c.titleEntry    -row 3 -column 0 -columnspan 2 -sticky w
        grid $c.subtitleLabel -row 5 -column 0 -columnspan 2 -sticky w
        grid $c.subtitleEntry -row 6 -column 0 -columnspan 2 -sticky w

        grid columnconfigure $c 0 -weight 0
        grid columnconfigure $c 1 -weight 1

        grid rowconfigure $c 0 -weight 0
        grid rowconfigure $c 1 -weight 0 -minsize 4
        grid rowconfigure $c 2 -weight 0
        grid rowconfigure $c 3 -weight 0
        grid rowconfigure $c 4 -weight 0 -minsize 4
        grid rowconfigure $c 5 -weight 0
        grid rowconfigure $c 6 -weight 0
        grid rowconfigure $c 7 -weight 1
    }

    proc previewWizard {} {

        set wizardCode [generateCode .wizard]

        catch {destroy .wizard}
        eval $wizardCode
        wm geometry .wizard 350x200
        if {[catch {.wizard show} error]} {
            tk_messageBox -icon info \
                -message "There was a problem displaying the wizard: $error" \
                -parent .wizthing
        }
    }

    proc generateWizard {} {
        variable wizData

        catch {destroy .wizshower}
        toplevel .wizshower
        text .wizshower.t -yscrollcommand {.wizshower.vsb set} \
            -font {-family Courier -size 10}
        scrollbar .wizshower.vsb -command {.wizshower.t yview}
        pack .wizshower.vsb -side right -fill y
        pack .wizshower.t -side left -fill both -expand y

	      .wizshower.t insert end [generateCode .wizard]
    }


    proc generateCode {w} {
       variable wizData

			global table proclist

			set proclist {}
			set x $table(rows)
			set y $table(cols)
			upvar #0 $table(array) f
			for {set i 0} {$i < $x} {incr i} {
				if {[string compare $f($i,1) ""] != 0} {
					set pair [list $f($i,1) 2]
					lappend proclist $pair
				}
			}

			 array set ::param [list lang $wizData(language) \
					platform $wizData(platform) api obj stub $wizData(stub)]

			 array set ::def [list \
					DLLDEFINE			{DLLEXPORT __declspec( dllexport )} \
					DEFINE				{} \
					STD_INCLUDE		[split $wizData(stdinclude)] \
					CUR_INCLUDE		[split $wizData(userinclude)] \
					PACKAGE			[list $wizData(package) $wizData(version)] \
					NAMESPACE			[list $wizData(namespace)] \
					PROCLIST			$proclist]

			return [wizard ::param]
    }
}

# adjust to center
set sw [winfo screenwidth .]
set sh [winfo screenheight .]
set width 450
set height 350
set orgx [expr ($sw-$width)/2]
set orgy [expr ($sh-$height)/2]

wm geometry .wizthing ${width}x${height}+$orgx+$orgy
wm minsize .wizthing $width $height

# set the initial order of the steps for this wizard. Not strictly
# necessary, but it helps document my intent
.wizthing order init step1 step2 finish

.wizthing show
